import os
from flask import Flask, request, jsonify
import psycopg2
from psycopg2 import Error
app = Flask(__name__)


RDS_HOST = "<RELLERNAR AQUI>" #TODO: Rellenar la variable de entorno RDS_HOST 
RDS_PASS = "superpass"

@app.route('/')
def index():
    return 'App Works!'

@app.route('/initialize')
def initialize_endpoint():
    db_params = {
        'host': RDS_HOST,
        'database': 'postgres',
        'user': 'postgres',
        'password': RDS_PASS
    }

    exception = None
    connection = None
    device_info = None

    try:
        # Establish the connection
        print("Connecting to the PostgreSQL database...")

        connection = psycopg2.connect(**db_params)

        print("Connection successful!")

        # Create a cursor to interact with the database
        cursor = connection.cursor()
        print("Cursor created.")

        # Execute a simple query

        cursor.execute("""
            CREATE TABLE IF NOT EXISTS devices (
                id SERIAL PRIMARY KEY,
                mac_address VARCHAR(20) NOT NULL UNIQUE,
                device_name VARCHAR(30) NOT NULL,
                creation_date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP
            );
        """)

        connection.commit()

        print("Table created successfully.")

        #Add a new device with a fixed mac address and a device name
        cursor.execute("INSERT INTO devices (mac_address, device_name) VALUES (%s, %s)", ('00:00:00:00:00:01', 'testing device'))
        connection.commit()
        print("New device added successfully.")

        cursor.execute("SELECT * FROM devices")
        device_info = cursor.fetchone()
        print("Added device:", device_info)


    except Error as e:
        print("Error:", e)
        exception = e
    finally:
        # Close the cursor and connection
        if connection:
            cursor.close()
            connection.close()
            print("Connection closed.")
    if exception:
        return f"Internal Server Error: {str(exception)}", 500

    return f'Initialitzation successful! \n Device info: \n {device_info}', 200

@app.route('/mqtt_messages')
def mqtt_messages_endpoint():

    mac = request.args.get('mac', default=None)  # Get the 'mac' parameter from the request

    if mac is None:
        return "Missing 'mac' parameter in the request."

    #DEBUG:
    print("mac: " + mac)

    exception = None
    connection = None

    # Database connection parameters
    db_params = {
        'host': RDS_HOST,
        'database': 'postgres',
        'user': 'postgres',
        'password': RDS_PASS
    }

    try:
        # Establish the connection
        print("Connecting to the PostgreSQL database...")

        connection = psycopg2.connect(**db_params)

        print("Connection successful!")

        # Create a cursor to interact with the database
        cursor = connection.cursor()
        print("Cursor created.")

        # Execute a simple query
        cursor.execute("SELECT * FROM devices WHERE devices.mac_address = %s", (mac,))
        device_info = cursor.fetchone()
        print("Connected to:", device_info)

    except Error as e:
        print("Error:", e)
        exception = e
    finally:
        # Close the cursor and connection
        if connection:
            cursor.close()
            connection.close()
            print("Connection closed.")
        if exception:
            return f"Internal Server Error: {str(exception)}", 500


    file_contents = None

    try:
        # Open the file in read mode
        with open("mqtt_messages.txt", 'r', encoding="utf-8") as file:
            # Read the contents of the file
            file_contents = file.read()
    except FileNotFoundError:
        print("The file mqtt_messages.txt does not exist.")
        file_contents = "Empty file"
    except Exception as e:
        print(f"An error occurred: {e}")
        return e

    return jsonify({"mac": mac, "file_contents": file_contents, "device_info": device_info})

if __name__ == '__main__':
    app.run(host="0.0.0.0", port=5000, debug=True)
